from thonny.plugins.micropython import MicroPythonProxy, MicroPythonConfigPage,\
    add_micropython_backend
from thonny import get_workbench, get_runner
import os
import subprocess
from thonny.ui_utils import SubprocessDialog
from thonny.running import get_frontend_python
from time import sleep

class ESPProxy(MicroPythonProxy):
    def _finalize_repl(self):
        # In some cases there may be still something coming.
        sleep(0.1)
        remainder = self._serial.read_all().decode("utf-8", "replace").strip()
        # display it unless it looks like an extra raw prompt
        if remainder and (len(remainder) > 40 or "raw REPL; CTRL-B to exit" not in remainder):
            self._send_error_to_shell(remainder)
            
    @property
    def firmware_filetypes(self):
        return [('*.bin files', '.bin'), ('all files', '.*')]
    
    def erase_flash(self):
        self.disconnect()
        cmd = [get_frontend_python(), '-u', '-m', 
                'esptool', 
                '--port', self.port, 
                'erase_flash']
        proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
                            universal_newlines=True)
        dlg = SubprocessDialog(get_workbench(), proc, "Erasing flash", autoclose=False)
        dlg.wait_window()
    
    @property
    def allow_webrepl(self):
        return True
        
        
class ESP8266Proxy(ESPProxy):
    description = "MicroPython on ESP8266"
    config_page_constructor = "ESP8266"
    
    @property
    def known_usb_vids_pids(self):
        return super().known_usb_vids_pids | {
            # eg. Adafruit Feather Huzzah 
            (0x10C4, 0xEA60), # Silicon Labs CP210x USB to UART Bridge,
            (0x1A86, 0x7523), # USB-SERIAL CH340,
        } 
    
    @property
    def flash_mode(self):
        # "dio" for some boards with a particular FlashROM configuration (e.g. some variants of a NodeMCU board)
        # https://docs.micropython.org/en/latest/esp8266/esp8266/tutorial/intro.html
        # https://github.com/espressif/esptool/wiki/SPI-Flash-Modes
        
        # TODO: detect the need for this (or provide conf option for the backend)
        return "keep"
        #return "dio"
        
    def construct_firmware_upload_command(self, firmware_path):
        return [get_frontend_python(), '-u', '-m', 
                'esptool', 
                '--port', self.port, 
                #'--baud', '460800',  
                'write_flash', 
                #'--flash_size', 'detect',
                #"--flash_mode", self.flash_mode,
                '0x0000', firmware_path]
        
    def _get_api_stubs_path(self):
        return os.path.join(os.path.dirname(__file__), "esp8266_api_stubs")
    
class ESP32Proxy(ESPProxy):
    @property
    def known_usb_vids_pids(self):
        return super().known_usb_vids_pids | {
            # eg. ESP-WROOM-32
            (0x10C4, 0xEA60), # Silicon Labs CP210x USB to UART Bridge
        }
        
    def construct_firmware_upload_command(self, firmware_path):
        return [get_frontend_python(), '-u', '-m', 
                'esptool', 
                #'--chip', 'esp32',
                '--port', self.port, 
                #'--baud', '460800', 
                'write_flash', 
                #'--flash_size=detect',
                '0x1000',
                firmware_path]

    def _get_api_stubs_path(self):
        return os.path.join(os.path.dirname(__file__), "esp32_api_stubs")

class ESP32LoBoProxy(ESP32Proxy):
    def _get_path_prefix(self):
        return "/flash/"        
        

    
class ESP8266ConfigPage(MicroPythonConfigPage):
    pass

class ESP32ConfigPage(MicroPythonConfigPage):
    pass

class ESP32LoBoConfigPage(ESP32ConfigPage):
    pass

def load_plugin():
    add_micropython_backend("ESP8266", ESP8266Proxy, "MicroPython (ESP8266)", ESP8266ConfigPage)
    add_micropython_backend("ESP32", ESP32Proxy, "MicroPython (ESP32)", ESP32ConfigPage)

    def upload_micropython():
        proxy = get_runner().get_backend_proxy()
        proxy.select_and_upload_micropython()
    
    def upload_micropython_enabled():
        proxy = get_runner().get_backend_proxy()
        return (getattr(proxy, "micropython_upload_enabled", False)
                and isinstance(proxy, ESPProxy))
        
    def erase_flash():
        proxy = get_runner().get_backend_proxy()
        proxy.erase_flash()
    
    def erase_flash_enabled():
        return isinstance(get_runner().get_backend_proxy(), ESPProxy) and get_runner().get_backend_proxy().micropython_upload_enabled
        
    get_workbench().add_command("uploadmicropythonesp", "device", "Install MicroPython to ESP8266/ESP32 ...",
                                upload_micropython,
                                upload_micropython_enabled,
                                group=40)

    get_workbench().add_command("erasespflash", "device", "Erase ESP8266/ESP32 flash",
                                erase_flash,
                                tester=erase_flash_enabled,
                                group=40)
    


CLIENT_ID = None

class MQTTClient:
    ''
    def _recv_len():
        pass

    def _send_str():
        pass

    def check_msg():
        pass

    def connect():
        pass

    def disconnect():
        pass

    def ping():
        pass

    def publish():
        pass

    def set_callback():
        pass

    def set_last_will():
        pass

    def subscribe():
        pass

    def wait_msg():
        pass


class Pin:
    ''
    IN = 0
    IRQ_FALLING = 2
    IRQ_RISING = 1
    OPEN_DRAIN = 2
    OUT = 1
    PULL_UP = 1
    def init():
        pass

    def irq():
        pass

    def off():
        pass

    def on():
        pass

    def value():
        pass

SERVER = '192.168.1.35'
TOPIC = None
led = None
machine = None
def main():
    pass

micropython = None
state = 0
def sub_cb():
    pass

ubinascii = None



LOG_DEBUG = 4
LOG_ERROR = 1
LOG_INFO = 3
LOG_NONE = 0
LOG_VERBOSE = 5
LOG_WARNING = 2
def dht_readinto():
    pass

def flash_erase():
    pass

def flash_read():
    pass

def flash_size():
    pass

def flash_user_start():
    pass

def flash_write():
    pass

def gpio_matrix_in():
    pass

def gpio_matrix_out():
    pass

def neopixel_write():
    pass

def osdebug():
    pass





class ULP:
    ''
    RESERVE_MEM = 2040
    def load_binary():
        pass

    def run():
        pass

    def set_wakeup_period():
        pass

WAKEUP_ALL_LOW = None
WAKEUP_ANY_HIGH = None
def wake_on_ext0():
    pass

def wake_on_ext1():
    pass

def wake_on_touch():
    pass





class APA106:
    ''
    ORDER = None
    def fill():
        pass

    def write():
        pass


class NeoPixel:
    ''
    ORDER = None
    def fill():
        pass

    def write():
        pass





class LockType:
    ''
    def acquire():
        pass

    def locked():
        pass

    def release():
        pass

def allocate_lock():
    pass

def exit():
    pass

def get_ident():
    pass

def stack_size():
    pass

def start_new_thread():
    pass




SET_CHARGE_PUMP = 141
SET_COL_ADDR = 33
SET_COM_OUT_DIR = 192
SET_COM_PIN_CFG = 218
SET_CONTRAST = 129
SET_DISP = 174
SET_DISP_CLK_DIV = 213
SET_DISP_OFFSET = 211
SET_DISP_START_LINE = 64
SET_ENTIRE_ON = 164
SET_MEM_ADDR = 32
SET_MUX_RATIO = 168
SET_NORM_INV = 166
SET_PAGE_ADDR = 34
SET_PRECHARGE = 217
SET_SEG_REMAP = 160
SET_VCOM_DESEL = 219

class SSD1306:
    ''
    def blit():
        pass

    def contrast():
        pass

    def fill():
        pass

    def fill_rect():
        pass

    def hline():
        pass

    def init_display():
        pass

    def invert():
        pass

    def line():
        pass

    def pixel():
        pass

    def poweroff():
        pass

    def poweron():
        pass

    def rect():
        pass

    def scroll():
        pass

    def show():
        pass

    def text():
        pass

    def vline():
        pass


class SSD1306_I2C:
    ''
    def blit():
        pass

    def contrast():
        pass

    def fill():
        pass

    def fill_rect():
        pass

    def hline():
        pass

    def init_display():
        pass

    def invert():
        pass

    def line():
        pass

    def pixel():
        pass

    def poweroff():
        pass

    def poweron():
        pass

    def rect():
        pass

    def scroll():
        pass

    def show():
        pass

    def text():
        pass

    def vline():
        pass

    def write_cmd():
        pass

    def write_data():
        pass


class SSD1306_SPI:
    ''
    def blit():
        pass

    def contrast():
        pass

    def fill():
        pass

    def fill_rect():
        pass

    def hline():
        pass

    def init_display():
        pass

    def invert():
        pass

    def line():
        pass

    def pixel():
        pass

    def poweroff():
        pass

    def poweron():
        pass

    def rect():
        pass

    def scroll():
        pass

    def show():
        pass

    def text():
        pass

    def vline():
        pass

    def write_cmd():
        pass

    def write_data():
        pass

def const():
    pass

framebuf = None




class NotFoundError:
    ''
def _makedirs():
    pass

def cleanup():
    pass

cleanup_files = None
debug = None
errno = None
def expandhome():
    pass

def fatal():
    pass

file_buf = None
gc = None
def get_install_path():
    pass

def get_pkg_metadata():
    pass

gzdict_sz = 31
def help():
    pass

def install():
    pass

install_path = None
def install_pkg():
    pass

def install_tar():
    pass

json = None
def main():
    pass

def op_basename():
    pass

def op_split():
    pass

os = None
def save_file():
    pass

sys = None
tarfile = None
def url_open():
    pass

usocket = None
ussl = None
uzlib = None
warn_ussl = None



CONFIG = './webrepl_cfg.py'
RC = './boot.py'
def change_daemon():
    pass

def exists():
    pass

def get_daemon_status():
    pass

def getpass():
    pass

def input_choice():
    pass

def input_pass():
    pass

machine = None
def main():
    pass

os = None
sys = None




class ADC:
    ''
    ATTN_0DB = 0
    ATTN_11DB = 3
    ATTN_2_5DB = 1
    ATTN_6DB = 2
    WIDTH_10BIT = 1
    WIDTH_11BIT = 2
    WIDTH_12BIT = 3
    WIDTH_9BIT = 0
    def atten():
        pass

    def read():
        pass

    def width():
        pass


class DAC:
    ''
    def write():
        pass

DEEPSLEEP = 4
DEEPSLEEP_RESET = 4
EXT0_WAKE = 1
EXT1_WAKE = 2
HARD_RESET = 2

class I2C:
    ''
    def init():
        pass

    def readfrom():
        pass

    def readfrom_into():
        pass

    def readfrom_mem():
        pass

    def readfrom_mem_into():
        pass

    def readinto():
        pass

    def scan():
        pass

    def start():
        pass

    def stop():
        pass

    def write():
        pass

    def writeto():
        pass

    def writeto_mem():
        pass

PIN_WAKE = 1

class PWM:
    ''
    def deinit():
        pass

    def duty():
        pass

    def freq():
        pass

    def init():
        pass

PWRON_RESET = 1

class Pin:
    ''
    IN = 1
    IRQ_FALLING = 2
    IRQ_RISING = 1
    OPEN_DRAIN = 7
    OUT = 3
    PULL_DOWN = 1
    PULL_UP = 0
    WAKE_HIGH = 5
    WAKE_LOW = 4
    def init():
        pass

    def irq():
        pass

    def value():
        pass


class RTC:
    ''
    def datetime():
        pass

    def init():
        pass

    def memory():
        pass

SLEEP = 2
SOFT_RESET = 5

class SPI:
    ''
    LSB = 1
    MSB = 0
    def deinit():
        pass

    def init():
        pass

    def read():
        pass

    def readinto():
        pass

    def write():
        pass

    def write_readinto():
        pass


class Signal:
    ''
    def off():
        pass

    def on():
        pass

    def value():
        pass

TIMER_WAKE = 3
TOUCHPAD_WAKE = 4

class Timer:
    ''
    ONE_SHOT = 0
    PERIODIC = 1
    def deinit():
        pass

    def init():
        pass

    def value():
        pass


class TouchPad:
    ''
    def config():
        pass

    def read():
        pass


class UART:
    ''
    def any():
        pass

    def init():
        pass

    def read():
        pass

    def readinto():
        pass

    def readline():
        pass

    def write():
        pass

ULP_WAKE = 5

class WDT:
    ''
    def feed():
        pass

WDT_RESET = 3
def deepsleep():
    pass

def disable_irq():
    pass

def enable_irq():
    pass

def freq():
    pass

def idle():
    pass

mem16 = None
mem32 = None
mem8 = None
def reset():
    pass

def reset_cause():
    pass

def sleep():
    pass

def time_pulse_us():
    pass

def unique_id():
    pass

def wake_reason():
    pass





class MQTTClient:
    ''
    DEBUG = None
    DELAY = 2
    def _recv_len():
        pass

    def _send_str():
        pass

    def check_msg():
        pass

    def connect():
        pass

    def delay():
        pass

    def disconnect():
        pass

    def log():
        pass

    def ping():
        pass

    publish = None
    reconnect = None
    def set_callback():
        pass

    def set_last_will():
        pass

    def subscribe():
        pass

    wait_msg = None
simple = None
utime = None



AF_INET = 2
AF_INET6 = 10
IPPROTO_IP = 0
IP_ADD_MEMBERSHIP = 1024
SOCK_DGRAM = 2
SOCK_RAW = 3
SOCK_STREAM = 1
SOL_SOCKET = 1
SO_REUSEADDR = 4
def callback():
    pass

def getaddrinfo():
    pass

def print_pcbs():
    pass

def reset():
    pass


class socket:
    ''
    def accept():
        pass

    def bind():
        pass

    def close():
        pass

    def connect():
        pass

    def listen():
        pass

    def makefile():
        pass

    def read():
        pass

    def readinto():
        pass

    def readline():
        pass

    def recv():
        pass

    def recvfrom():
        pass

    def send():
        pass

    def sendall():
        pass

    def sendto():
        pass

    def setblocking():
        pass

    def setsockopt():
        pass

    def settimeout():
        pass

    def write():
        pass




_webrepl = None
def accept_conn():
    pass

client_s = None
listen_s = None
network = None
def setup_conn():
    pass

socket = None
def start():
    pass

def start_foreground():
    pass

def stop():
    pass

uos = None
websocket = None
websocket_helper = None




class ADC:
    ''
    def read():
        pass

DEEPSLEEP = 4
DEEPSLEEP_RESET = 5
HARD_RESET = 6

class I2C:
    ''
    def init():
        pass

    def readfrom():
        pass

    def readfrom_into():
        pass

    def readfrom_mem():
        pass

    def readfrom_mem_into():
        pass

    def readinto():
        pass

    def scan():
        pass

    def start():
        pass

    def stop():
        pass

    def write():
        pass

    def writeto():
        pass

    def writeto_mem():
        pass


class PWM:
    ''
    def deinit():
        pass

    def duty():
        pass

    def freq():
        pass

    def init():
        pass

PWRON_RESET = 0

class Pin:
    ''
    IN = 0
    IRQ_FALLING = 2
    IRQ_RISING = 1
    OPEN_DRAIN = 2
    OUT = 1
    PULL_UP = 1
    def init():
        pass

    def irq():
        pass

    def off():
        pass

    def on():
        pass

    def value():
        pass


class RTC:
    ''
    ALARM0 = 0
    def alarm():
        pass

    def alarm_left():
        pass

    def datetime():
        pass

    def irq():
        pass

    def memory():
        pass

SOFT_RESET = 4

class SPI:
    ''
    LSB = 1
    MSB = 0
    def deinit():
        pass

    def init():
        pass

    def read():
        pass

    def readinto():
        pass

    def write():
        pass

    def write_readinto():
        pass


class Signal:
    ''
    def off():
        pass

    def on():
        pass

    def value():
        pass


class Timer:
    ''
    ONE_SHOT = 0
    PERIODIC = 1
    def deinit():
        pass

    def init():
        pass


class UART:
    ''
    def any():
        pass

    def init():
        pass

    def read():
        pass

    def readinto():
        pass

    def readline():
        pass

    def write():
        pass


class WDT:
    ''
    def deinit():
        pass

    def feed():
        pass

WDT_RESET = 1
def deepsleep():
    pass

def disable_irq():
    pass

def enable_irq():
    pass

def freq():
    pass

def idle():
    pass

mem16 = None
mem32 = None
mem8 = None
def reset():
    pass

def reset_cause():
    pass

def sleep():
    pass

def time_pulse_us():
    pass

def unique_id():
    pass




AP_IF = 1
AUTH_OPEN = 0
AUTH_WEP = 1
AUTH_WPA2_PSK = 3
AUTH_WPA_PSK = 2
AUTH_WPA_WPA2_PSK = 4
MODE_11B = 1
MODE_11G = 2
MODE_11N = 3
STAT_CONNECTING = 1
STAT_CONNECT_FAIL = 4
STAT_GOT_IP = 5
STAT_IDLE = 0
STAT_NO_AP_FOUND = 3
STAT_WRONG_PASSWORD = 2
STA_IF = 0
def WLAN():
    pass

def phy_mode():
    pass





class CancelledError:
    ''
DEBUG = 0

class EventLoop:
    ''
    def call_at_():
        pass

    def call_later():
        pass

    def call_later_ms():
        pass

    def call_soon():
        pass

    def close():
        pass

    def create_task():
        pass

    def run_forever():
        pass

    def run_until_complete():
        pass

    def stop():
        pass

    def time():
        pass

    def wait():
        pass


class IORead:
    ''
    def handle():
        pass


class IOReadDone:
    ''
    def handle():
        pass


class IOWrite:
    ''
    def handle():
        pass


class IOWriteDone:
    ''
    def handle():
        pass


class PollEventLoop:
    ''
    def add_reader():
        pass

    def add_writer():
        pass

    def call_at_():
        pass

    def call_later():
        pass

    def call_later_ms():
        pass

    def call_soon():
        pass

    def close():
        pass

    def create_task():
        pass

    def remove_reader():
        pass

    def remove_writer():
        pass

    def run_forever():
        pass

    def run_until_complete():
        pass

    def stop():
        pass

    def time():
        pass

    def wait():
        pass


class SleepMs:
    ''
    def handle():
        pass


class StopLoop:
    ''
    def handle():
        pass


class StreamReader:
    ''
    aclose = None
    read = None
    readexactly = None
    readline = None

class StreamWriter:
    ''
    aclose = None
    awrite = None
    awriteiter = None
    def get_extra_info():
        pass


class SysCall:
    ''
    def handle():
        pass


class SysCall1:
    ''
    def handle():
        pass

def Task():
    pass


class TimeoutError:
    ''

class TimeoutObj:
    ''
_socket = None
def cancel():
    pass

core = None
def coroutine():
    pass

def ensure_future():
    pass

def get_event_loop():
    pass

log = None
open_connection = None
select = None
def set_debug():
    pass

sleep = None
sleep_ms = None
start_server = None
time = None

class type_gen:
    ''
    def close():
        pass

    def pend_throw():
        pass

    def send():
        pass

    def throw():
        pass

uasyncio = None
ucollections = None
uerrno = None
utimeq = None
def wait_for():
    pass

wait_for_ms = None



DIRTYPE = 'dir'

class FileSection:
    ''
    def read():
        pass

    def readinto():
        pass

    def skip():
        pass

REGTYPE = 'file'
TAR_HEADER = None

class TarFile:
    ''
    def extractfile():
        pass

    def next():
        pass


class TarInfo:
    ''
def roundup():
    pass

uctypes = None




class Response:
    ''
    def close():
        pass

    content = None
    def json():
        pass

    text = None
def delete():
    pass

def get():
    pass

def head():
    pass

def patch():
    pass

def post():
    pass

def put():
    pass

def request():
    pass

usocket = None



from setuptools import setup
import os.path

setupdir = os.path.dirname(__file__)

requirements = []
for line in open(os.path.join(setupdir, 'requirements.txt'), encoding="UTF-8"):
    if line.strip() and not line.startswith('#'):
        requirements.append(line)

setup(
      name="thonny-esp",
      version="0.2",
      description="ESP8266 and ESP32 MicroPython support for Thonny IDE",
      long_description="""Plug-in for Thonny IDE which adds ESP8266 and ESP32 MicroPython backends. 
      
More info: 

* https://github.com/thonny/thonny-esp
* https://github.com/thonny/thonny/wiki/MicroPython
* https://thonny.org

NB! This plug-in is only relevant for Thonny up to 3.2.1. 
Since 3.2.2 this functionality is included in the main package.
""",
      url="https://github.com/thonny/thonny-esp/",
      author="Aivar Annamaa",
      author_email="aivar.annamaa@gmail.com",
      license="MIT",
      classifiers=[
        "Environment :: MacOS X",
        "Environment :: Win32 (MS Windows)",
        "Environment :: X11 Applications",
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "Intended Audience :: End Users/Desktop",
        "License :: Freeware",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: MacOS",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: POSIX",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Education",
        "Topic :: Software Development",
        "Topic :: Software Development :: Embedded Systems",
      ],
      keywords="IDE education programming ESP8266 ESP32 MicroPython Thonny",
      platforms=["Windows", "macOS", "Linux"],
      python_requires=">=3.5",
      include_package_data=True,
      package_data={'thonnycontrib.esp': ['esp8266_api_stubs/*', 'esp32_api_stubs/*']},
      install_requires=requirements,
      packages=["thonnycontrib.esp"],
)



class MQTTClient:
    ''
    def _recv_len():
        pass

    def _send_str():
        pass

    def check_msg():
        pass

    def connect():
        pass

    def disconnect():
        pass

    def ping():
        pass

    def publish():
        pass

    def set_callback():
        pass

    def set_last_will():
        pass

    def subscribe():
        pass

    def wait_msg():
        pass


class MQTTException:
    ''
def hexlify():
    pass

socket = None
struct = None



SLEEP_LIGHT = 1
SLEEP_MODEM = 2
SLEEP_NONE = 0
def apa102_write():
    pass

def check_fw():
    pass

def deepsleep():
    pass

def dht_readinto():
    pass

def esf_free_bufs():
    pass

def flash_erase():
    pass

def flash_id():
    pass

def flash_read():
    pass

def flash_size():
    pass

def flash_user_start():
    pass

def flash_write():
    pass

def free():
    pass

def freemem():
    pass

def info():
    pass

def malloc():
    pass

def meminfo():
    pass

def neopixel_write():
    pass

def osdebug():
    pass

def set_native_code_location():
    pass

def sleep_type():
    pass

